# -*- coding: utf-8 -*-
from setuphelpers import *
import win32con

r"""
Usable WAPT package functions: install(), uninstall(), session_setup(), audit(), update_package()

"""
# Declaring global variables - Warnings: 1) WAPT context is only available in package functions; 2) Global variables are not persistent between calls
bin_name = "SpotifyFullSetup.exe"
app_name = "Spotify"
app_dir = makepath(programfiles, app_name)
bin_path = makepath(app_dir, bin_name)


def install():
    # Installing the software
    mkdirs(app_dir)
    print("Copying: %s to: %s" % (bin_name, app_dir))
    filecopyto(bin_name, app_dir)


def uninstall():
    # Uninstalling the software
    if isdir(app_dir):
        remove_tree(app_dir)


def session_setup():
    # Declaring local variables
    package_version = control.version.split("-")[0]
    user_reg_version = ".".join(
        registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Spotify", "DisplayVersion").split(".")[:4]
    )
    user_app_dir = makepath(user_appdata, app_name)
    user_app_path = makepath(user_app_dir, "Spotify.exe")

    # Removing Spotify Appx
    remove_user_appx("SpotifyAB.SpotifyMusic")

    # Determining to install the software or not based on user agreement
    if not registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "Install") or force:
        if params.get("install_with_luti", False):
            ask_to_install = 6
        else:
            ask_to_install = messagebox(
                "Configuring %s" % app_name, "Do you want to install %s on this user account?" % app_name, style=win32con.MB_YESNOCANCEL
            )
        if ask_to_install == 6:
            registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "Install", "Yes")
        elif ask_to_install == 7:
            registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "Install", "No")

    # Installing or Uninstalling the software in user environment
    if registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "Install") == "Yes":
        if package_version > user_reg_version or user_reg_version == "":
            print("Installing: %s (%s) in user environment" % (bin_name, package_version))
            install_exe_if_needed(bin_path, silentflags="/silent", min_version=package_version)
    elif registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "Install") == "No":
        if isfile(user_app_path):
            print("Uninstalling: %s from user environment" % (app_name))
            run('"%s" %s' % (user_app_path, "/uninstall /silent"))
        registry_delete(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "DisableAutorunOnce")

    # Disabling app autorun
    if not reg_value_exists(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "DisableAutorunOnce"):
        registry_delete(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", app_name)
        registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\WAPT\%s" % app_name, "DisableAutorunOnce", "Done")


def remove_user_appx(package):
    r"""Remove Windows AppX package from user environment

    Args:
        package (str): AppX Package name, asterisk character (*) can be used as wildcard

    .. versionadded:: 2.3

    """
    run_powershell('Get-AppxPackage "%s" | Remove-AppxPackage' % package)
